 
#include <cv.h>
#include <highgui.h>
#include <stdio.h>
#include <math.h>

int uniform(uchar pixel, float param) {
  double value = (rand()/(double)(RAND_MAX/2) - 1.0);
  int newpixel = pixel + value*255*param;
  newpixel = newpixel > 255 ? 255 : newpixel;
  newpixel = newpixel < 0 ? 0 : newpixel;      
  return newpixel;
}

int poisson(uchar pixel) {
    double L = exp(-(double)pixel);
    double p = 1.0;
    int k = 0;
    do {
      k++;
      p = p * (rand()/(double)(RAND_MAX));
    } while (p > L);
    
    int newpixel = (int)k-1;
    newpixel = newpixel > 255 ? 255 : newpixel;
    newpixel = newpixel < 0 ? 0 : newpixel;      
    return newpixel;  
}


int gaussian(uchar pixel, float param) {
  float x1, x2, w, y;
  do {
    x1 = 2.0 * (rand()/(double)(RAND_MAX)) - 1.0;
    x2 = 2.0 * (rand()/(double)(RAND_MAX)) - 1.0;
    w = (x1*x1)+(x2*x2);
  } while (w >= 1.0);
  w = sqrt( (-2.0* log(w)) / w);
  y = (x1+x2)/2 * w;
  int newpixel = pixel + (int)(y*255*param);
  newpixel = newpixel > 255 ? 255 : newpixel;
  newpixel = newpixel < 0 ? 0 : newpixel;      
  return newpixel;
}


void generateNoise(IplImage * img, int type, float param) 
{
      uchar * data = (uchar *)img->imageData;
      int width = img->width;
      int height = img->height;
      int step = img->widthStep;
      int i, j;
      
      for (i=0; i<height; i++) {
	  for (j=0; j<width; j++) {
		switch(type) {
		  case 1: // uniform distributed noise
			  data[i*step+j] = (int)uniform(data_n[i*step+j], param);
			  break;
		  case 2: // gaussian distributed noise
			  data[i*step+j] = (int)gaussian(data_n[i*step+j], param);
			  break;
		  case 3: // poisson distributed noise
			  data[i*step+j] = (int)poisson(data_n[i*step+j]);
			  break;
		}
	  }	  
      }
}

int main(int argc, char *argv[]) {
 
  IplImage * img, * img_n;
  
  const char * filename;
  int type = 0;
  float param = 0.0;
  
  filename = argv[1];
  type = atoi(argv[2]);
  param = atof(argv[3]);
  
  img = cvLoadImage( filename, CV_LOAD_IMAGE_GRAYSCALE );
  if (!img)
     return -1;
  
  // create empty image with size of img and pixedl depth = 8
  img_n = cvCreateImage( cvGetSize(img), IPL_DEPTH_8U, 1);
  
  // copy img to img_n
  cvCopy(img, img_n, NULL);
  
  denoiseMeanPoint(img, img_d, param);
  
  cvNamedWindow("Original",0);
  cvNamedWindow("Restored",0);
  cvShowImage("Original", img);
  cvShowImage("Restored", img_d);
  cvWaitKey(0);
  
  // store image
  cvSaveImage("denoised.jpg",img_d,0);
  
  // free memory
  cvReleaseImage(&img);
  cvReleaseImage(&img_n);  
}

